package com.jpattern.orm.generator.statement;

import java.io.InputStream;
import java.io.Reader;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Date;
import java.sql.NClob;
import java.sql.RowId;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.HashMap;
import java.util.Map;

/**
 * 
 * @author ufo
 *
 */
public class PreparedStatementManipulatorFactory {

	private Map<Class<?>, PreparedStatementManipulator<?>> manipulators = new HashMap<Class<?>, PreparedStatementManipulator<?>>();
	private PreparedStatementManipulator<?> defaultManipulator = new ObjectPreparedStatementManipulator();

	public PreparedStatementManipulatorFactory() {
		addManipulator(InputStream.class, new InputStreamPreparedStatementManipulator());
		addManipulator(Reader.class, new ReaderPreparedStatementManipulator());
		addManipulator(Blob.class, new BlobPreparedStatementManipulator());
		addManipulator(Clob.class, new ClobPreparedStatementManipulator());
		addManipulator(String.class, new StringPreparedStatementManipulator());
		addManipulator(byte[].class, new ByteArrayPreparedStatementManipulator());
		addManipulator(Date.class, new DatePreparedStatementManipulator());
		addManipulator(NClob.class, new NClobPreparedStatementManipulator());
		addManipulator(RowId.class, new RowIdPreparedStatementManipulator());
		addManipulator(Time.class, new TimePreparedStatementManipulator());
		addManipulator(Timestamp.class, new TimestampPreparedStatementManipulator());

	}

	public <T> PreparedStatementManipulator<?> getPreparedStatementManipulator(final Class<T> clazz) {
		if (this.manipulators.containsKey(clazz)) {
			return this.manipulators.get(clazz);
		}
		return this.defaultManipulator;
	}

	private <T> void addManipulator(final Class<T> type, final PreparedStatementManipulator<T> manipulator) {
		this.manipulators.put(type, manipulator);
	}
}
